#define F_CPU 16000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h> 
#include "time.h"
#define TCNT_START  6
#include<stdlib.h>
#define SWITCH_PC3_PRESSED() ( ( PINC & (1<<PC3) ) == 0 )
#define SWITCH_PC4_PRESSED() ( ( PINC & (1<<PC4) ) == 0 )
#define SWITCH_PC5_PRESSED() ( ( PINC & (1<<PC5) ) == 0 )
static volatile uint32_t millis;

void init_newperi() {
	DDRC |= (1<<PC2) | (1<<PC1) | (1<<PC0);
	DDRC &= ~( (1<<PC3 ) | (1<<PC4) | (1<<PC5) );
	PORTC |= (1<<PC3);
	PORTC |= (1<<PC4);
	PORTC |= (1<<PC5);
	DDRB |= (1 << PB4) | (1<<PB3) | (1<<PB2) | (1<<PB1) | (1<<PB0);
}

void set_led_mole(uint8_t value) {
	PORTC &= ~(0b111);
        	PORTC |= (value & 0b111);
}
void set_led_stage(uint8_t stage) {
	PORTB &= ~(0b11111);
	PORTB |= (stage & 0b11111);
}

void timer_init()
{
    TCCR0A = 0x00;

    // Clock source = ClkIO/64
    // => 250 KHz
    TCCR0B = (0<<CS02)|(1<<CS01)|(1<<CS00);

    // Enable Timer0 overflow interrupt
    TIMSK0 = (1<<TOIE0);

    // Start counting with desired overflow interval
    // so that it will overflow (>255) in 250 ticks
    TCNT0 = TCNT_START;
}

uint32_t timer_millis()
{
    uint32_t t;
	uint8_t saved_sreg = SREG;

    cli();
    t = millis;
    SREG = saved_sreg;

    return t;
}

int main() {
	timer_init();
	init_newperi();	
	int n;
	int n3;
	uint8_t value;
	uint8_t zero = 0;
	int count = 0;
	int sta = 0;
	 uint8_t stage = 1;
	int check = 0;
	static uint32_t ts = 0;
	_delay_ms(2000);
	while(1) {
			
		n = rand();
    		n3 = n%3;
		if (n3 == 0) {
      			value = 1;
   		 } else if (n3 == 1) {
      			value = 2;
    		} else if (n3 == 2) {
      			value = 4;
    		}
		if (n3 == 0) {
			_delay_ms(20);
			set_led_mole(value);
			_delay_ms(500);
			while(1) {
				if (SWITCH_PC3_PRESSED() && !SWITCH_PC4_PRESSED() && !SWITCH_PC5_PRESSED() ) {
					count++;
					ts = timer_millis();
					break;
				} else if ( SWITCH_PC4_PRESSED() || SWITCH_PC5_PRESSED() ) {
					check++;
					break;
				}
				if ( timer_millis()-ts < 2000-(100*sta)) {
					check++;
					break;
				}
			}		
			_delay_ms(20);
		} else if (n3 == 1) {
			_delay_ms(20);
			set_led_mole(value);
			_delay_ms(500);
			while(1) {
				if (SWITCH_PC4_PRESSED() && !SWITCH_PC3_PRESSED() && !SWITCH_PC5_PRESSED() ) {
					count++;
					ts = timer_millis();
					break;
				} else if ( SWITCH_PC3_PRESSED() || SWITCH_PC5_PRESSED() ) {
					check++;
					break;
				}
				if ( timer_millis()-ts < 2000-(100*sta)) {
					check++;
					break;
				}
			}
			_delay_ms(20);
		} else if (n3 == 2) {
			_delay_ms(20);
			set_led_mole(value);
			_delay_ms(500);
			while(1) {
				if (SWITCH_PC5_PRESSED() && !SWITCH_PC3_PRESSED() && !SWITCH_PC4_PRESSED() ) {
					count++;
					ts = timer_millis();
					break;
				} else if ( SWITCH_PC3_PRESSED() || SWITCH_PC4_PRESSED() ) {
					check++;
					break;
				}
				if ( timer_millis()-ts < 2000-(100*sta)) {
					check++;
					break;
				}
			}
			_delay_ms(20);
		}
		if (count == 5) {
			count = 0;
			sta++;
			stage++;
		}
		if (sta == 31) {
			set_led_stage(stage);
			break;
		}
		if (check != 0) {
			set_led_stage(stage);
			break;
		}
		set_led_mole(zero);
		_delay_ms(1000);
	}
}